package org.bouncycastle.graphics.rt;

import javax.vecmath.*;

public class BBox
{
    public float    minX, minY, minZ;
    public float    maxX, maxY, maxZ;

    float    t1, t2;

    public BBox()
    {
        set(0, 0, 0, 1, 1, 1);
    }

    public BBox(
        float    x1,
        float    y1,
        float    z1,
        float    x2,
        float    y2,
        float    z2)
    {
        set(x1, y1, z1, x2, y2, z2);
    }

    public BBox(
        BBox    bBox)
    {
        set(bBox);
    }

    public BBox add(
        BBox    bBox)
    {
        BBox    box = new BBox(this);

        box.adjustFor(bBox);

        return box;
    }

    public BBox intersects(
        BBox    bBox)
    {
        BBox    box = new BBox(this);

        if (bBox.minX > box.minX)
        {
            box.minX = bBox.minX;
        }

        if (bBox.maxX < box.maxX)
        {
            box.maxX = bBox.maxX;
        }

        if (bBox.minY > box.minY)
        {
            box.minY = bBox.minY;
        }

        if (bBox.maxY < box.maxY)
        {
            box.maxY = bBox.maxY;
        }

        if (bBox.minZ > box.minZ)
        {
            box.minZ = bBox.minZ;
        }

        if (bBox.maxZ < box.maxZ)
        {
            box.maxZ = bBox.maxZ;
        }

        //
        // confirm we have a sensible result.
        //
        if ((box.maxX < box.minX)
           || (box.maxY < box.minY)
           || (box.maxZ < box.minZ))
        {
            return null;
        }
        
        return box;
    }

    public void set(
        BBox    bBox)
    {
        this.minX = bBox.minX;
        this.minY = bBox.minY;
        this.minZ = bBox.minZ;
        this.maxX = bBox.maxX;
        this.maxY = bBox.maxY;
        this.maxZ = bBox.maxZ;
    }

    public void set(
        float    x1,
        float    y1,
        float    z1,
        float    x2,
        float    y2,
        float    z2)
    {
        if (x1 < x2)
        {
            minX = x1;
            maxX = x2;
        }
        else
        {
            minX = x2;
            maxX = x1;
        }

        if (y1 < y2)
        {
            minY = y1;
            maxY = y2;
        }
        else
        {
            minY = y2;
            maxY = y1;
        }

        if (z1 < z2)
        {
            minZ = z1;
            maxZ = z2;
        }
        else
        {
            minZ = z2;
            maxZ = z1;
        }
    }

    public void adjustFor(
        float    x,
        float    y,
        float    z)
    {
        if (x < minX)
        {
            minX = x;
        }
        else if (x > maxX)
        {
            maxX = x;
        }

        if (y < minY)
        {
            minY = y;
        }
        else if (y > maxY)
        {
            maxY = y;
        }

        if (z < minZ)
        {
            minZ = z;
        }
        else if (z > maxZ)
        {
            maxZ = z;
        }
    }

    public void adjustFor(
        Point3f    p)
    {
        if (p.x < minX)
        {
            minX = p.x;
        }
        else if (p.x > maxX)
        {
            maxX = p.x;
        }

        if (p.y < minY)
        {
            minY = p.y;
        }
        else if (p.y > maxY)
        {
            maxY = p.y;
        }

        if (p.z < minZ)
        {
            minZ = p.z;
        }
        else if (p.z > maxZ)
        {
            maxZ = p.z;
        }
    }

    /**
     * expand a bounding box by value in every direction on every axis.
     */
    public void expand(
        double  value)
    {
        minX -= value;
        minY -= value;
        minZ -= value;
        maxX += value;
        maxY += value;
        maxZ += value;
    }

    public BBox getTransformedBBox(
        Matrix4f    mat)
    {
        BBox    bb;
        Point3f    c, p;

        p = new Point3f(minX, minY, minZ);

        mat.transform(p);

        bb = new BBox(p.x, p.y, p.z, p.x, p.y, p.z);

        p.set(maxX, maxY, maxZ);
        mat.transform(p);
        bb.adjustFor(p);

        p.set(minX, maxY, maxZ);
        mat.transform(p);
        bb.adjustFor(p);

        p.set(maxX, minY, maxZ);
        mat.transform(p);
        bb.adjustFor(p);

        p.set(maxX, maxY, minZ);
        mat.transform(p);
        bb.adjustFor(p);

        p.set(minX, minY, maxZ);
        mat.transform(p);
        bb.adjustFor(p);

        p.set(minX, maxY, minZ);
        mat.transform(p);
        bb.adjustFor(p);

        p.set(maxX, minY, minZ);
        mat.transform(p);
        bb.adjustFor(p);

        return bb;
    }

    public void adjustFor(
        BBox    bb)
    {
        if (bb.minX < minX)
        {
            minX = bb.minX;
        }
        if (bb.maxX > maxX)
        {
            maxX = bb.maxX;
        }

        if (bb.minY < minY)
        {
            minY = bb.minY;
        }
        if (bb.maxY > maxY)
        {
            maxY = bb.maxY;
        }

        if (bb.minZ < minZ)
        {
            minZ = bb.minZ;
        }
        if (bb.maxZ > maxZ)
        {
            maxZ = bb.maxZ;
        }
    }

    private final boolean clipToPlanes(
        float    org,
        float    dir,
        float    min,
        float    max,
        float[]    tvals)
    {
        if (dir != 0)
        {
            float    t1, t2, b1, b2;

            if (dir < 0)
            {
                b2 = min - org;
                if (b2 > 0)
                {
                    return false;
                }
                b1 = max - org;
            }
            else
            {
                b2 = max - org;
                if (b2 < 0)
                {
                    return false;
                }
                b1 = min - org;
            }

            if ((t2 = (b2 / dir)) < tvals[0])
            {
                return false;
            }
            
            if ((t1 = (b1 / dir)) > tvals[1])
            {
                return false;
            }

            if (t2 < tvals[1])
            {
                tvals[1] = t2;
            }

            if (t1 > tvals[0])
            {
                tvals[0] = t1;
            }
        }
        else if (org < min || org > max)
        {
            return false;
        }

        return true;
    }

    /**
     * checks a ray defined by the combination of org and dir against a
     * bounding box returning true if it intersects false otherwise. The
     * tvals array is set with the entry and exit distances of the ray.
     * 
     * @param org the origin of the ray.
     * @param dir the direction of the ray.
     * @param tvals return array for the distances to the entry and exit 
     *    points on the box.
     */
    public boolean isIntersecting(
        Point3f         org,
        Vector3f        dir,
        float[]         tvals)
    {
        tvals[0] = 0;
        tvals[1] = Float.MAX_VALUE;

        if (clipToPlanes(org.x, dir.x, minX, maxX, tvals))
        {
            if (clipToPlanes(org.y, dir.y, minY, maxY, tvals))
            {
                if (clipToPlanes(
                    org.z, dir.z, minZ, maxZ, tvals))
                {
                    return true;
                }
            }
        }

        return false;
    }

    public boolean contains(
        Point3f    p)
    {
        if (p.x < minX || p.x > maxX)
        {
            return false;
        }

        if (p.y < minY || p.y > maxY)
        {
            return false;
        }

        if (p.z < minZ || p.z > maxZ)
        {
            return false;
        }

        return true;
    }

    public boolean isMissing(
        Point3f         org,
        Vector3f        dir)
    {
        float[]    tVals = new float[2];

        tVals[0] = 0;
        tVals[1] = Float.MAX_VALUE;

        if (clipToPlanes(org.x, dir.x, minX, maxX, tVals))
        {
            if (clipToPlanes(org.y, dir.y, minY, maxY, tVals))
            {
                if (clipToPlanes(
                    org.z, dir.z, minZ, maxZ, tVals))
                {
                    return false;
                }
            }
        }

        return true;
    }

    public boolean isMissedBy(
        Ray    r)
    {
        float[]    tVals = new float[2];

        tVals[0] = 0;
        tVals[1] = r.theHit.t;

        if (clipToPlanes(r.org.x, r.dir.x, minX, maxX, tVals))
        {
            if (clipToPlanes(r.org.y, r.dir.y, minY, maxY, tVals))
            {
                if (clipToPlanes(
                    r.org.z, r.dir.z, minZ, maxZ, tVals))
                {
                    return false;
                }
            }
        }

        return true;
    }

    public String toString()
    {
        return "[" + minX + ", " + minY + ", " + minZ + ", "
                + maxX + ", " + maxY + ", " + maxZ + "]";
    }
}
