package org.bouncycastle.graphics.rt;

import javax.vecmath.*;

import java.io.*;
import java.util.*;

import org.bouncycastle.graphics.rt.util.*;
import org.bouncycastle.graphics.rt.tools.io.*;

/**
 * The background object
 * <p>
 * Example:
 * <code>
 * <pre>
 *    &lt;Background color="0, 0, 0"/&gt;
 * </pre>
 * </code>
 * <p>
 * Document Type Definition:
 * <code>
 * <pre>
 *     &lt;!ENTITY %Color3f "CDATA"&gt;
 *
 *     &lt;!ELEMENT Background&gt;
 *     &lt;!ATTLIST Background
 *         color  %Color3f; "0, 0, 0"&gt;
 * </pre>
 * </code>
 */
public class Background
{
    float        r, g, b;

    Shader      shader;

    String      name = "default";

    public Background()
    {
    }

    public static Background getInstance(
        RMLReader   in)
        throws IOException
    {
        Background  b = new Background();

        ParametersMap   params = in.parameters();

        b.setColor(params.getColor("color", "0, 0, 0"));

        if (in.token() != RMLReader.EMPTY_TAG)
        {
            in.skipToEndTag(in.toString());
        }

        return b;
    }

    public String getName()
    {
        return name;
    }

    public void setColor(
        Color3f c)
    {
        this.r = c.x;
        this.g = c.y;
        this.b = c.z;
    }

    public void setColor(
        double    r,
        double    g,
        double    b)
    {
        this.r = (float)r;
        this.g = (float)g;
        this.b = (float)b;
    }

    public void setShader(
        Shader    shader)
    {
        this.shader = shader;
    }

    public void shade(
        Ray         ray,
        Pixel     p)
    {
        if (shader != null)
        {
            throw new RuntimeException("Ooops!");
        }

        p.set(r, g, b);
    }
}
