package org.bouncycastle.graphics.rt;

import javax.vecmath.*;

public class Pixel
{
    public float    r, g, b, a;

    public final void set(
        float    r,
        float    g,
        float    b,
        float    a)
    {
        this.r = r;
        this.g = g;
        this.b = b;
        this.a = a;
    }

    public final void set(
        float    r,
        float    g,
        float    b)
    {
        this.r = r;
        this.g = g;
        this.b = b;
        this.a = 1;
    }

    public final void set(
        Tuple3f    p)
    {
        r = p.x;
        g = p.y;
        b = p.z;
        this.a = 1;
    }

    public final void add(
        Pixel    p)
    {
        r += p.r;
        g += p.g;
        b += p.b;
        a += p.a;
    }

    public final void add(
        Tuple3f    col)
    {
        r += col.x;
        g += col.y;
        b += col.z;
    }

    public final void scaledAdd(
        float    scale,
        Pixel    p)
    {
        r += scale * p.r;
        g += scale * p.g;
        b += scale * p.b;
        a += scale * p.a;
    }

    public final void scaledAdd(
        Pixel    scale,
        Tuple3f    col)
    {
        r += scale.r * col.x;
        g += scale.g * col.y;
        b += scale.b * col.z;
    }

    public final void scaledAdd(
        Tuple3f    scale,
        Tuple3f    col)
    {
        r += scale.x * col.x;
        g += scale.y * col.y;
        b += scale.z * col.z;
    }

    public final void scaledAdd(
        Tuple3f    scale1,
        Tuple3f    scale2,
        Tuple3f    col)
    {
        r += scale1.x * scale2.x * col.x;
        g += scale1.y * scale2.y * col.y;
        b += scale1.z * scale2.z * col.z;
    }

    public final void scaledAdd(
        Tuple3f    scale1,
        Tuple3f    scale2,
        Pixel    col)
    {
        r += scale1.x * scale2.x * col.r;
        g += scale1.y * scale2.y * col.g;
        b += scale1.z * scale2.z * col.b;
    }

    public final void scaledAdd(
        Pixel    scale1,
        Tuple3f    scale2,
        Tuple3f    col)
    {
        r += scale1.r * scale2.x * col.x;
        g += scale1.g * scale2.y * col.y;
        b += scale1.b * scale2.z * col.z;
    }

    public final void scaledAdd(
        Tuple3f    scale,
        Pixel    p)
    {
        r += scale.x * p.r;
        g += scale.y * p.g;
        b += scale.z * p.b;
    }

    public final void scaledAdd(
        float    scale,
        Tuple3f    p)
    {
        r += scale * p.x;
        g += scale * p.y;
        b += scale * p.z;
    }

    public final void scale(
        float    scale)
    {
        r *= scale;
        g *= scale;
        b *= scale;
        a *= scale;
    }

    public String toString()
    {
        return "(" + r + ", " + g + ", " + b + ", " + a + ")";
    }
}
