package org.bouncycastle.graphics.rt;

import javax.vecmath.*;

public class Ray
{
    public boolean      done;
    public int          id;
    public int          level;
    public int          timeSlice;
    public float        tMax;
    public Color3f      col = new Color3f();
    public Point3f      org = new Point3f();
    public Vector3f     dir = new Vector3f();
    public Renderable   orgObj;
    public Hit          theHit = new Hit(),
                        hit = new Hit();

    private Ray         next;

    private static Ray    rayList = null;

    private static int    rayNumber = 0;

    private boolean       disposed = false;

    private Ray()
    {
        theHit.i = this;
        hit.i = this;
    }

    public static Ray getInstance()
    {
        Ray    r;

        if (rayList == null)
        {
            r = new Ray();
        }
        else
        {
            r = rayList;
            rayList = rayList.next;
        }

        r.theHit.t = Float.MAX_VALUE;
        r.theHit.obj = null;
        r.done = false;
        r.level = 0;
        r.id = rayNumber++;
        r.tMax = Float.MAX_VALUE;
        r.orgObj = null;
        r.disposed = false;

        return r;
    }

    public void dispose()
    {
        if (this.disposed)
        {
            throw new IllegalStateException("attempt to free ray already freed");
        }

        this.disposed = true;
        this.next = rayList;
        rayList = this;
    }

    public void setHit(
        Hit         other)
    {
        hit.t = other.t;
        hit.obj = other.obj;
        hit.colorCalc = other.colorCalc;
        hit.normCalc = other.normCalc;
        hit.stMapper = other.stMapper;
        hit.shader = other.shader;
        hit.loc.set(other.loc);
        hit.normalSet = other.normalSet;
        hit.subtractedSurface = other.subtractedSurface;
    }

    public void setHit(
        Renderable          obj,
        float               t,
        ColorCalculator     colorCalc,
        NormalCalculator    normCalc,
        TextureMapper       stMapper,
        Shader              shader)
    {
        hit.t = t;
        hit.obj = obj;
        hit.colorCalc = colorCalc;
        hit.normCalc = normCalc;
        hit.stMapper = stMapper;
        hit.shader = shader;
        hit.loc.scaleAdd(t, dir, org);
    }

    public void setTheHit()
    {
        theHit.t = hit.t;
        theHit.obj = hit.obj;
        theHit.colorCalc = hit.colorCalc;
        theHit.normCalc = hit.normCalc;
        theHit.stMapper = hit.stMapper;
        theHit.shader = hit.shader;
        theHit.loc.set(hit.loc);
        theHit.normalSet = false;
        theHit.stSet = false;
        theHit.objLocSet = false;
    }

    public boolean trace(
        Traceable    obj,
        Pixel        pix)
    {
        return trace(obj, null, pix);
    }

    public boolean trace(
        Traceable     obj,
        Background    background,
        Pixel         pix)
    {
        pix.set(0, 0, 0, 0);

        if (obj.isIntersected(this))
        {
            theHit.shader.shade(theHit, pix);

            return true;
        }

        if (background != null)
        {
            background.shade(this, pix);
        }

        return false;
    }

    public String toString()
    {
        return "[" + org + "," + dir + "]"
            + "\n" + "orgObj = " + orgObj
            + " tMax = " + tMax;
    }
}
