package org.bouncycastle.graphics.rt;

public class World
    extends Renderable
    implements Illuminated
{
    Illuminated     objs = null;

    float           base;
    float           totalArea;

    Camera          camera;

    public void setObjects(
        Illuminated   objs)
    {
        this.objs = objs;
    }

    void setCamera(
        Camera      camera)
    {
        this.camera = camera;
    }

    void setScreenSize(
        int     width,
        int     height)
    {
        base = (width < height) ? (width / 2) : (height / 2);
        totalArea = width * height;
    }

    /**
     * return a rough estimate of how big a given object would be
     * in screen space, given it's bounding box, and how far away
     * it is from the eyepoint.
     */
    public float calculateScreenSize(
        double  distance,
        BBox    bBox)
    {
        float   edge;
        float   viewFactor = camera.viewFactor();

        //
        // find largest face
        //
        if ((bBox.maxZ - bBox.minZ) > (bBox.maxY - bBox.minY))
        {
            if ((bBox.maxZ - bBox.minZ) > (bBox.maxX - bBox.minX))
            {
                edge = bBox.maxZ - bBox.minZ;
            }
            else
            {
                edge = bBox.maxX - bBox.minX;
            }
        }
        else 
        {
            if ((bBox.maxY - bBox.minY) > (bBox.maxX - bBox.minX))
            {
                edge = bBox.maxY - bBox.minY;
            }
            else
            {
                edge = bBox.maxX - bBox.minX;
            }
        }
        
        return (float)((base * edge) / (distance / viewFactor) * ((base * edge) / (distance / viewFactor)));
    }

    public Illuminated getObjects()
    {
        return objs;
    }

    public boolean isShadowing(
        Ray   r)
    {
        return objs.isShadowing(r);
    }

    public boolean isIntersected(
        Ray r)
    {
        return objs.isIntersected(r);
    }

    public HitList intersections(
        Ray r)
    {
        return objs.intersections(r);
    }

    public LightDataList visibleLights(
        Hit hit)
    {
        return objs.visibleLights(hit);
    }
}
