package org.bouncycastle.graphics.rt.mappers;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.*;

public class ConicalMap
    implements TextureMapper
{
    public static TextureMapper canonicalMap = new ConicalMap();

    private float    tipVal = 0;

    public ConicalMap()
    {
    }

    public ConicalMap(
        double    r1,
        double    r2)
    {
        if (r1 > r2)
        {
            tipVal = (float)(r2 / r1);
        }
        else
        {
            tipVal = (float)(r1 / r2);
        }
    }

    /*
     * generate a planar mapping from a surface onto a tile. In this case
     * we calculate for a cone running along the z axis.
     */
    public boolean getST(
        Tuple3f    loc,
        float[]    uv)
    {
        int        w, h, indx;

        if (Math.abs(loc.x) > 1)
        {
            loc.x = (loc.x < 0) ? -1 : 1;
        }

        uv[T] = 1 - (Math.abs(loc.z) - tipVal) / (1 - tipVal);
        if (loc.z != 0)
        {
            float    fact;
        
            fact = loc.x / loc.z;
            if (Math.abs(fact) > 1)
            {
                fact = (fact < 0) ? -1 : 1;
            }

            uv[S] = (float)(Math.acos(fact) / (2.0 * Math.PI));
        }
        else
        {
            uv[S] = 1;
        }

        uv[T] = (loc.z < 0) ? 1 - uv[T] : uv[T];
        uv[S] = (loc.y < 0) ? 1 - uv[S] : uv[S];

        return true;
    }
}
