package org.bouncycastle.graphics.rt.mappers;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.*;

public class SphericalMap
    implements TextureMapper
{
    public static TextureMapper    canonicalMap = new SphericalMap();

    /**
     * return the (s, t) values for the inverse mapping of a canonical
     * sphere given the point on the spheres surface.
     */
    public boolean getST(
        Tuple3f        l,
        float[]        st)
    {
        float        t;
        float        phi, theta;

        t = -l.z;

        if (Math.abs(t) > 1)        // precision check
        {
            t = (t < 0) ? -1 : 1;
        }

        phi = (float)Math.acos(t);

        st[T] = (float)(phi / Math.PI);

        if (st[T] != 0 && st[T] != 1)
        {
            Vector3f    loc = new Vector3f();

            loc.x = l.x;
            loc.y = l.y;
            loc.z = 0;

            loc.normalize();

            if (Math.abs(loc.x) > 1)    // precision check 
            {
                loc.x = (loc.x < 0) ? -1 : 1;
            }

            theta = (float)(Math.acos(loc.x) / (2 * Math.PI));

            if (loc.y > 0.0)
            {
                st[S] = theta;
            }
            else
            {
                st[S] = 1 - theta;
            }
        }
        else
        {
            st[S] = 0;
        }

        return true;
    }
}
