package org.bouncycastle.graphics.rt.mappers;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.*;

public class ToroidalMap
    implements TextureMapper
{
    private float    factor;

    public ToroidalMap(
        double    ringR,
        double    pipeR)
    {
        factor = 1 - (float)((pipeR / ringR) * (pipeR / ringR));
    }

    public boolean getST(
        Tuple3f    l,
        float[]    uv)
    {
        float        fact;
        Vector3f    tmp = new Vector3f(l.x, l.y, 0);
        Vector3f    loc = new Vector3f(l);
                         
        tmp.normalize();

        if (Math.abs(tmp.x) > 1)
        {
            tmp.x = (tmp.x < 0) ? -1 : 1;
        }
        
        if (tmp.y < 0)
        {
            uv[S] = 1 - (float)(Math.acos(tmp.x) / (2 * Math.PI));
        }
        else
        {
            uv[S] = (float)(Math.acos(tmp.x) / (2 * Math.PI));
        }

                        /* align n->x with x */
        loc.x = loc.x * tmp.x + loc.y * tmp.y;

        fact = loc.x * loc.x + loc.z * loc.z + factor;

        loc.x = loc.x * (fact - 2);
        loc.y = 0;
        loc.z = loc.z * fact;

        loc.normalize();

        if (Math.abs(loc.x) > 1)
        {
            loc.x = (loc.x < 0) ? -1 : 1;
        }

        if (loc.z < 0)
        {
            uv[T] = 1 - (float)(Math.acos(loc.x) / (2 * Math.PI));
        }
        else
        {
            uv[T] = (float)(Math.acos(loc.x) / (2 * Math.PI));
        }

        return true;
    }
}
