package org.bouncycastle.graphics.rt.objects;

import java.io.*;
import java.util.*;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.tools.io.*;

/**
 * a list of colors.
 * <p>
 * Example:
 * <code>
 * <pre>
 *        &lt;Colors&gt;
 *            0.000, 1.000, 0.000,
 *            1.000, 1.000, 0.000,
 *            1.000, 1.000, 1.000,
 *            1.000, 0.000, 0.000
 *        &lt;/Colors&gt;
 * </pre>
 * </code>
 * <p>
 * Document Type Definition:
 * <code>
 * <pre>
 *     &lt;!ELEMENT Colors (#PCDATA)&gt;
 * </pre>
 * </code>
 */
public class Colors
{
    Color3f[]  cols;

    private Colors()
    {
    }

    public static Colors getInstance(
        RMLReader   in)
        throws IOException
    {
        Colors cs = new Colors();

        if (in.readNext() != RMLReader.EOF)
        {
            StringTokenizer st = new StringTokenizer(in.toString(), ", \t");
    
            cs.cols = new Color3f[st.countTokens() / 3];

            for (int i = 0; i != cs.cols.length; i++)
            {
                Color3f    c = new Color3f();

                c.x = Float.parseFloat(st.nextToken());
                c.y = Float.parseFloat(st.nextToken());
                c.z = Float.parseFloat(st.nextToken());

                cs.cols[i] = c;
            }
        }
        else
        {
            throw new IOException("unexpected EOF in Colors tag");
        }

        //
        // swallow the end tag.
        //
        if (in.readNext() == RMLReader.EOF)
        {
            throw new IOException("unexpected EOF in Colors tag");
        }

        return cs;
    }

    public Color3f[] colors()
    {
        return cols;
    }
}
