package org.bouncycastle.graphics.rt.objects;

import java.io.*;
import java.util.*;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.tools.io.*;

/**
 * a list of normals.
 * <p>
 * Example:
 * <code>
 * <pre>
 *        &lt;Normals&gt;
 *            0.000, 1.000, 0.000,
 *            0.000, 1.000, 0.000,
 *            0.000, 0.000, 1.000,
 *            1.000, 0.000, 0.000
 *        &lt;/Normals&gt;
 * </pre>
 * </code>
 * <p>
 * Document Type Definition:
 * <code>
 * <pre>
 *     &lt;!ELEMENT Normals (#PCDATA)&gt;
 * </pre>
 * </code>
 */
public class Normals
{
    Vector3f[]  norms;

    private Normals()
    {
    }

    public static Normals getInstance(
        RMLReader   in)
        throws IOException
    {
        Normals ns = new Normals();

        if (in.readNext() != RMLReader.EOF)
        {
            StringTokenizer st = new StringTokenizer(in.toString(), ", \t");
    
            ns.norms = new Vector3f[st.countTokens() / 3];

            for (int i = 0; i != ns.norms.length; i++)
            {
                Vector3f    n = new Vector3f();

                n.x = Float.parseFloat(st.nextToken());
                n.y = Float.parseFloat(st.nextToken());
                n.z = Float.parseFloat(st.nextToken());

                ns.norms[i] = n;
            }
        }
        else
        {
            throw new IOException("unexpected EOF in Normals tag");
        }

        //
        // swallow the end tag.
        //
        if (in.readNext() == RMLReader.EOF)
        {
            throw new IOException("unexpected EOF in Normals tag");
        }

        return ns;
    }

    public Vector3f[] normals()
    {
        return norms;
    }
}
