package org.bouncycastle.graphics.rt.objects;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.*;

/**
 * foundation class for the basic solids.
 */
public abstract class SimpleObject
    extends Bounded
{
    Vector3f            clip = null;

    Vector3f            v = new Vector3f();

    float               tolerance = 0;

    float                t1, t2;
    NormalCalculator     enNorm, exNorm;
    Shader               enShade, exShade;
    TextureMapper            enMap, exMap;


    BBox        bb;

    public SimpleObject()
    {
    }

    public boolean isShadowing(
        Ray     r)
    {
        if (r.orgObj == this && !this.isSelfShadowing())
        {
            return false;
        }

        Ray     nr = this.createObjectSpaceRay(r);

        if (!isObjectIntersected(nr))
        {
            return false;
        }

        nr.dispose();

        if (t1 > tolerance)
        {
            return (t1 < r.tMax);
        }

        return (t2 < r.tMax);
    }

    public HitList intersections(
        Ray     r)
    {
        Ray     nr = this.createObjectSpaceRay(r);

        if (!isObjectIntersected(nr))
        {
            return null;
        }

        HitList list = null;

        if (t1 > tolerance)
        {
            list = HitList.getInstance();

            list.add(r, this, t1, this, enNorm, enMap, exShade);
            list.add(r, this, t2, this, exNorm, exMap, exShade);
        }
        else if (t2 > tolerance)
        {
            list = HitList.getInstance();

            list.add(r, this, t2, this, exNorm, exMap, exShade);
        }

        nr.dispose();

        return list;
    }

    public boolean isIntersected(
        Ray    r)
    {
        Ray     nr = this.createObjectSpaceRay(r);

        if (!isObjectIntersected(nr))
        {
            return false;
        }

        if (t1 > tolerance)
        {
            r.setHit(this, t1, this, enNorm, enMap, enShade);
            return true;
        }
        else if (t2 > tolerance)
        {
            r.setHit(this, t2, this, exNorm, exMap, exShade);
            return true;
        }

        nr.dispose();

        return false;
    }

    protected float doClip(
        float        t,
        Point3f        org,
        Vector3f    dir)
    {
        v.set(org);
        v.scaleAdd(t, dir);

        if (v.x < 0.0 || v.y < 0.0)
        {
            return 0;
        }

        if (clip.dot(v) < 0.0)
        {
            return 0;
        }

        return t;
    }

    protected abstract boolean isObjectIntersected(Ray r);
}
