package org.bouncycastle.graphics.rt.objects;

import java.io.*;
import java.util.*;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.tools.io.*;

/**
 * a list of vertices.
 * <p>
 * Example:
 * <code>
 * <pre>
 *        &lt;Vertices&gt;
 *            0.000, 1.000, 0.000,
 *            1.000, 1.000, 0.000,
 *            0.000, 0.000, 0.000,
 *            1.000, 0.000, 0.000
 *        &lt;/Vertices&gt;
 * </pre>
 * </code>
 * <p>
 * Document Type Definition:
 * <code>
 * <pre>
 *     &lt;!ELEMENT Vertices (#PCDATA)&gt;
 * </pre>
 * </code>
 */
public class Vertices
{
    Point3f[]  verts;

    private Vertices()
    {
    }

    public static Vertices getInstance(
        RMLReader   in)
        throws IOException
    {
        Vertices vs = new Vertices();

        if (in.readNext() != RMLReader.EOF)
        {
            StringTokenizer st = new StringTokenizer(in.toString(), ", \t");
    
            vs.verts = new Point3f[st.countTokens() / 3];

            for (int i = 0; i != vs.verts.length; i++)
            {
                Point3f    v = new Point3f();

                v.x = Float.parseFloat(st.nextToken());
                v.y = Float.parseFloat(st.nextToken());
                v.z = Float.parseFloat(st.nextToken());

                vs.verts[i] = v;
            }
        }
        else
        {
            throw new IOException("unexpected EOF in Vertices tag");
        }

        //
        // swallow the end tag.
        //
        if (in.readNext() == RMLReader.EOF)
        {
            throw new IOException("unexpected EOF in Vertices tag");
        }

        return vs;
    }

    public Point3f[] vertices()
    {
        return verts;
    }
}
