package org.bouncycastle.graphics.rt.shaders.ct;

import java.io.*;

import javax.vecmath.*;

import org.bouncycastle.graphics.rt.*;
import org.bouncycastle.graphics.rt.tools.io.*;
import org.bouncycastle.graphics.rt.util.ParametersMap;
import org.bouncycastle.graphics.rt.shaders.CookTorranceShader;

/**
 * An ambient only surface shader.
 * <p>
 * Example:
 * <code>
 * <pre>
 *      &lt;Ambient ambient="0.9"/&gt;
 * </pre>
 * </code>
 * <p>
 * Document Type Definition:
 * <code>
 * <pre>
 *     &lt;!ENTITY %Float    "CDATA"&gt;
 *     &lt;!ENTITY %Color3f  "CDATA"&gt;
 *
 *     &lt;!ELEMENT Ambient EMPTY&gt;
 *     &lt;!ATTLIST Ambient
 *         ambient           (%Color3f; | %Float;) "0, 0, 0"&gt;
 * </pre>
 * </code>
 */
public class Ambient
    extends CookTorranceShader
{
    public Ambient(
        World world)
    {
        super(world);
    }

    public static Shader getInstance(
        World         world,
        RMLReader       in)
        throws IOException
    {
        Ambient   a = new Ambient(world);

        a.readObject(in);

        return a;
    }

    public void shade(
        Hit      hit,
        Pixel    p)
    {
        p.set(0, 0, 0, 1);

        p.scaledAdd(hit.color(), ambientContribution(world));

        p.scale(alpha);
    }
}
