package org.bouncycastle.graphics.rt.tools;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * a display program for pix files.
 * <p>
 * usage: java org.bouncycastle.graphics.rt.tools.Disp file
 * <p>
 * where file is a ".pix" file. 
 */
public class Disp
{
    private static class Display
        extends JFrame
    {
        MemoryImageSource   source;
        Image               image;

        Display(
            String  imageName,
            int     width,
            int     height,
            int[]   pix)
        {
            super("Disp: " + imageName);

            source = new MemoryImageSource(width, height, pix, 0, width);
            image = createImage(source);

            setSize(width, height);
            setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

            this.addWindowListener(new WindowAdapter()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        System.exit(0);
                    }
                });
        }

        public void paint(
            Graphics    g)
        {
            g.drawImage(image, 0, 0, this);
        }
    }

    public static void main(
        String[]    args)
        throws IOException
    {
        int             argc = 0;
        BufferedInputStream  in;
        String          imageName = "";

        if (argc != args.length)
        {
            try
            {
                in = new BufferedInputStream(new FileInputStream(args[argc]));
                imageName = args[argc];
            }
            catch (FileNotFoundException e)
            {
                System.err.println("can't open file " + args[argc]);
                System.exit(1);
                return;
            }
        }
        else
        {
            in = new BufferedInputStream(System.in);
        }

        String  type = readLine(in);
        String  dimensions = readLine(in);
        int[]   pix;
        int     width, height;

        if (type.equals("RGB"))
        {
            StringTokenizer st = new StringTokenizer(dimensions, " ");

            width = Integer.parseInt(st.nextToken());
            height = Integer.parseInt(st.nextToken());

            int depth = Integer.parseInt(st.nextToken());

            pix = new int[width * height];

            for (int i = 0; i != pix.length; i++)
            {
                pix[i] = (255 << 24) | (in.read()  << 16) | (in.read() << 8) | (in.read());
            }
        }
        else
        {
            throw new IOException("Unknown image type");
        }

        Display display = new Display(imageName, width, height, pix);

        display.show();
    }

	static final byte LF = 0x0A;
	static final byte CR = 0x0D;

	static public String readLine(BufferedInputStream in) throws IOException
	{
		String s = new String();
		int b;

		do {
			b = in.read();
			if (b != LF && b != CR) s = s + (char)b;
			if (b == CR) {
				in.mark(1);	// this will always succeed for buffered input streams
				b = in.read();
				if (b != LF) {
					// we've found a lone CR, reset buffer to before char of next line
					in.reset();
					}
				}
			} while (b != LF && b != CR);

		return s;
	}
}
