package org.bouncycastle.graphics.rt.tools;

import java.io.*;

import org.bouncycastle.graphics.rt.*;
import org.bouncycastle.graphics.rt.tools.io.*;

/**
 * a parser for RML files.
 * <p>
 * usage: java org.bouncycastle.graphics.rt.tools.Parser file
 * <p>
 * where file is a ".rml" file. This will produce a file with
 * with a ".pix" extension that has the same base name as the
 * ".rml" file.
 */
public class Parser
{
    public static void main(
        String[]    args)
    {
        int         argc = 0;
        int         tok;
        RMLReader   in;
        String      outFile = "out.pix";

        if (argc != args.length)
        {
            try
            {
                in = new RMLReader(new FileReader(args[argc]));
                outFile = args[argc].substring(0, args[argc].lastIndexOf(".rml")) + ".pix";
            }
            catch (FileNotFoundException e)
            {
                System.err.println("can't open file " + args[argc]);
                System.exit(1);
                return;
            }
        }
        else
        {
            in = new RMLReader(new InputStreamReader(System.in));
        }

        try
        {
            Renderer renderer = (Renderer)in.readObject();

            renderer.setClamp(true);

            FileOutputStream   out = new FileOutputStream(outFile);
            BufferedWriter     wOut = new BufferedWriter(new OutputStreamWriter(out));

            wOut.write("RGB", 0, 3);
            wOut.newLine();
    
            String hdr =  renderer.getImageWidth()
                           + " " + renderer.getImageHeight() + " 8 8 8";
            wOut.write(hdr, 0, hdr.length());
            wOut.newLine();
            wOut.flush();

            System.out.println("Rendering...");
            renderer.doRender(Renderer.RGB, out);
        } 
        catch (IOException e)
        {
            System.err.println("IOException: " + e.getMessage());
            System.exit(1);
            return;
        }

        System.exit(0);
    }
}
