package org.bouncycastle.graphics.rt.tools;

import java.io.*;
import java.util.*;

/**
 * Program to convert a pix file into a ppm file.
 * <p>
 * usage: java org.bouncycastle.graphics.rt.tools.PixToPPM file
 * <p>
 * where file is a ".pix" file. This will produce a file with
 * with a ".ppm" extension that has the same base name as the
 * ".pix" file.
 */
public class PixToPPM
{
    private static String readLine(
        InputStream   in)
        throws IOException
    {
        StringBuffer    sb = new StringBuffer();
        int             c;

        while ((c = in.read()) != '\n')
        {
            if (c == '\r')
            {
                continue;
            }

            sb.append((char)c);
        }

        return sb.toString();
    }

    public static void main(
        String[]    args)
        throws IOException
    {
        int                 argc = 0;
        InputStream         in;
        String              outFile = "out.ppm";

        if (argc != args.length)
        {
            try
            {
                in = new FileInputStream(args[argc]);
                outFile = args[argc].substring(0, args[argc].lastIndexOf(".pix")) + ".ppm";

            }
            catch (FileNotFoundException e)
            {
                System.err.println("can't open file " + args[argc]);
                System.exit(1);
                return;
            }
        }
        else
        {
            in = System.in;
        }

        FileOutputStream   out = new FileOutputStream(outFile);
        BufferedWriter     wOut = new BufferedWriter(new OutputStreamWriter(out));
        String              type = readLine(in);
        String              dimensions = readLine(in);
        int[]               pix;
        int                 width, height;

        if (type.equals("RGB"))
        {
            StringTokenizer st = new StringTokenizer(dimensions, " ");

            width = Integer.parseInt(st.nextToken());
            height = Integer.parseInt(st.nextToken());

            int depth = Integer.parseInt(st.nextToken());

            wOut.write("P6", 0, 2);
            wOut.newLine();

            String  hdr = width + " " + height;

            wOut.write(hdr, 0, hdr.length());
            wOut.newLine();
            wOut.write("255", 0, 3);
            wOut.newLine();

            wOut.flush();

            int     len;
            byte[]  buf = new byte[width * 3];

            while ((len = in.read(buf, 0, buf.length)) >= 0)
            {
                out.write(buf, 0, len);
            }

            out.close();
        }
        else
        {
            throw new IOException("Unknown image type");
        }
    }
}
