package org.bouncycastle.graphics.rt.util;

import javax.vecmath.*;

public class SampleMask
{
    static Point2f[] points2x2 =
        { 
            new Point2f((float)-0.25, (float)0.25),
            new Point2f((float)0.25, (float)-0.25),
            new Point2f((float)-0.25, (float)-0.25),
            new Point2f((float)0.25, (float)0.25),
        };

    static Point2f[] points3x3 =
        { 
            new Point2f((float)0.0, (float)0.0),
            new Point2f((float)(-2 / 6.0), (float)(2 / 6.0)),
            new Point2f((float)(-2 / 6.0), (float)(-2 / 6.0)),
            new Point2f((float)(-2 / 6.0), (float)0.0),
            new Point2f((float)(2 / 6.0), (float)(2 / 6.0)),
            new Point2f((float)0.0, (float)(2 / 6.0)),
            new Point2f((float)(2 / 6.0), (float)(-2 / 6.0)),
            new Point2f((float)0.0, (float)(-2 / 6.0)),
            new Point2f((float)0.0, (float)(2 / 6.0))
        };

    static Point2f[] points4x4 =
        { 
            new Point2f((float)(-3 / 8.0), (float)(3 / 8.0)),
            new Point2f((float)(-3 / 8.0), (float)(1 / 8.0)),
            new Point2f((float)(-3 / 8.0), (float)(-1 / 8.0)),
            new Point2f((float)(-3 / 8.0), (float)(-3 / 8.0)),
            new Point2f((float)(-1 / 8.0), (float)(3 / 8.0)),
            new Point2f((float)(-1 / 8.0), (float)(1 / 8.0)),
            new Point2f((float)(-1 / 8.0), (float)(-1 / 8.0)),
            new Point2f((float)(-1 / 8.0), (float)(-3 / 8.0)),
            new Point2f((float)(1 / 8.0), (float)(3 / 8.0)),
            new Point2f((float)(1 / 8.0), (float)(1 / 8.0)),
            new Point2f((float)(1 / 8.0), (float)(-1 / 8.0)),
            new Point2f((float)(1 / 8.0), (float)(-3 / 8.0)),
            new Point2f((float)(3 / 8.0), (float)(3 / 8.0)),
            new Point2f((float)(3 / 8.0), (float)(1 / 8.0)),
            new Point2f((float)(3 / 8.0), (float)(-1 / 8.0)),
            new Point2f((float)(3 / 8.0), (float)(-3 / 8.0))
        };

    static Point2f[] circle5Points =
        {
            new Point2f((float)0.0, (float)0.0),
            new Point2f((float)0.0, (float)0.5),
            new Point2f((float)0.0, (float)-0.5),
            new Point2f((float)0.5, (float)0.0),
            new Point2f((float)-0.5, (float)0.0)
        };

    static Point2f[] circle7Points =
        {
            new Point2f((float)0.0, (float)0.0),
            new Point2f((float)0.000000, (float)0.666667),
            new Point2f((float)0.000000, (float)-0.666667),
            new Point2f((float)0.577350, (float)0.333333),
            new Point2f((float)-0.577350, (float)0.333333),
            new Point2f((float)0.577350, (float)-0.333333),
            new Point2f((float)-0.577350, (float)-0.333333)
        };

    static Point2f[] circle13Points =
        {
            new Point2f((float)0.0, (float)0.0),
            new Point2f((float)0.333333, (float)0.577350),        /* outer */
            new Point2f((float)0.666667, (float)0.000000),
            new Point2f((float)0.333333, (float)-0.577350),
            new Point2f((float)-0.333333, (float)-0.577350),
            new Point2f((float)-0.666667, (float)0.000000),
            new Point2f((float)-0.333333, (float)0.577350),
            new Point2f((float)0.000000, (float)0.333333),        /* inner */
            new Point2f((float)0.577350, (float)0.166667),
            new Point2f((float)0.577350, (float)-0.166667),
            new Point2f((float)0.000000, (float)-0.333333),
            new Point2f((float)-0.577350, (float)-0.166667),
            new Point2f((float)-0.577350, (float)0.166667)
        };

    static Point2f[] circle19Points =
        {
            new Point2f((float)0.0, (float)0.0),
            new Point2f((float)0.750, (float)0.433),    /* outer */
            new Point2f((float)0.0, (float)0.866),
            new Point2f((float)-0.750, (float)0.433),
            new Point2f((float)-0.750, (float)-0.433),
            new Point2f((float)0.0, (float)-0.866),
            new Point2f((float)0.750, (float)-0.433),
            new Point2f((float)0.75, (float)0.0),
            new Point2f((float)0.375, (float)0.650),    /* inner */
            new Point2f((float)-0.375, (float)0.650),
            new Point2f((float)-0.750, (float)0.000),
            new Point2f((float)-0.375, (float)-0.650),
            new Point2f((float)0.375, (float)-0.650),
            new Point2f((float)0.375, (float)0.216),
            new Point2f((float)0.000, (float)0.433),
            new Point2f((float)-0.375, (float)0.217),
            new Point2f((float)-0.375, (float)-0.216),
            new Point2f((float)0.000, (float)-0.433),
            new Point2f((float)0.375, (float)-0.217)
        };

    public static SampleMask    sample2x2 = new SampleMask((float)0.25, points2x2);
    public static SampleMask    sample3x3 = new SampleMask((float)(2 / 6.0), points3x3);
    public static SampleMask    sample4x4 = new SampleMask((float).0125, points4x4);

    public static SampleMask    circle5 = new SampleMask((float)0.25, circle5Points);
    public static SampleMask    circle7 = new SampleMask((float)0.14, circle7Points);
    public static SampleMask    circle13 = new SampleMask((float)0.090, circle13Points);
    public static SampleMask    circle19 = new SampleMask((float)0.075, circle19Points);

    public float        magnitude;
    public Point2f[]    points;

    public SampleMask(
        float        magnitude,
        Point2f[]    points)
    {
        this.magnitude = magnitude;
        this.points = points;
    }
        
    public static SampleMask getSquareSampleMask(
        int    nsamples)
    {
        if (nsamples > 9)
            return sample4x4;

        if (nsamples > 4)
            return sample3x3;

        return sample2x2;
    }

    public static SampleMask getCircularSampleMask(
        int    nsamples)
    {
        if (nsamples >= 19)
            return circle19;

        if (nsamples >= 13)
            return circle13;

        if (nsamples >= 7)
            return circle7;

        return circle5;
    }
}
