package org.bouncycastle.graphics.rt.util;

import    javax.vecmath.*;

public class Util
{
    /**
     * generate a peturbed vector around axis, leaving the result in res.
     */
    public static void generatePerturbationVector(
        Vector3f    axis,
        float        u,
        float        v,
        Vector3f    res)
    {
        float        a, b, c, fact;

        a = axis.x;
        b = axis.y;
        c = axis.z;

        //
        // bump mapped polygons start out life with (0, 0, 1) as their
        // normal, as this happens a lot it's worth checking for!
        //
        if (a == 0.0 && b == 0.0)
        {
            fact = Math.abs(c);
        }
        else
        {
            fact = (float)Math.sqrt(b * b + c * c);
        }

        //
        // rotate the vector (u, v, 0) so that it aligns with
        // axis
        //
        if (fact != 0.0)
        {
            res.set(
                fact * u,
                (-a * b * u + c * v) / fact,
                (-a * c * u + b * v) / fact);
        }
        else
        {
            res.set(0, u, -v);
        }
    }

    /**
     * generate an array of size matrices based on the interpolated
     * values in the input matrix array.
     */
    public static Matrix4f[] interpolate(
        int         size,
        Matrix4f[]  input)
    {
        Matrix4f[]    results;
        float        interval;
        int        startIndex, endIndex;
        float        startInterval, endInterval;

        results = new Matrix4f[size];

        results[0] = new Matrix4f(input[0]);
        results[results.length - 1] = new Matrix4f(
                        input[input.length - 1]);

        interval = (results.length - 1) / (float)(input.length - 1);

        endInterval = interval;
        startInterval = 0;
        startIndex = 0;
        endIndex = 1;

        for (int i = 1; i != results.length - 1; i++)
        {
            Matrix4f    m1, m2;

            if (i > endInterval)
            {
                startInterval = endInterval;
                endInterval += interval;
                
                startIndex = endIndex;
                endIndex += 1;
            }

            m1 = input[startIndex];
            m2 = input[endIndex];
            
            float        scale1 = (endInterval - i) / interval;
            float        scale2 = (i - startInterval) / interval;

            results[i] = new Matrix4f(
                    m1.m00 * scale1 + m2.m00 * scale2,
                    m1.m01 * scale1 + m2.m01 * scale2,
                    m1.m02 * scale1 + m2.m02 * scale2,
                    m1.m03 * scale1 + m2.m03 * scale2,
                    m1.m10 * scale1 + m2.m10 * scale2,
                    m1.m11 * scale1 + m2.m11 * scale2,
                    m1.m12 * scale1 + m2.m12 * scale2,
                    m1.m13 * scale1 + m2.m13 * scale2,
                    m1.m20 * scale1 + m2.m20 * scale2,
                    m1.m21 * scale1 + m2.m21 * scale2,
                    m1.m22 * scale1 + m2.m22 * scale2,
                    m1.m23 * scale1 + m2.m23 * scale2,
                    m1.m30 * scale1 + m2.m30 * scale2,
                    m1.m31 * scale1 + m2.m31 * scale2,
                    m1.m32 * scale1 + m2.m32 * scale2,
                    m1.m33 * scale1 + m2.m33 * scale2);
        }

        return results;
    }

    /*
     * Produces a triangular wave....
     */
    static public float triWave(
        float   x)
    {
        float    a = 1;

        //
        // Get X into the range 0 to 1
        //
        if (x < 0)
        {
            a = -1;
            x = -x;
        }

        x -= (int)x;

        if (x < 0.25) 
        {
            return (a * x * 4);
        }
        else if (x < 0.75)
        {
             return a - 4 * a * (x - 0.25f);
        }

        return (4 * a * (x - 0.75f) - a);
    }
}
